###############################################################################
#	Module Name - QueryData.pm
#	Perl Interface to BigMac Dictionary
#	TEAM 		ALIANZA LIMA

#	Members 	Amine Abou-Rjeily
#			Amruta Purandare
#			Sam Storie
#			Suchitra Goopy

###############################################################################

#	Copyright (c) 2002,
#	Alianza Lima
#	Amruta Purandare, University of Minnesota, Duluth
#	pura0010@d.umn.edu
#	Suchitra Goopy, University of Minnesota, Duluth
#	goop0001@d.umn.edu
#	Amine Abou-Rjeily, University of Minnesota, Duluth
#	abou0013@d.umn.edu
#	Sam Storie, University of Minnesota, Duluth
#	sstorie@d.umn.edu
#	Ted Pedersen, University of Minnesota, Duluth
#	tpederse@umn.edu

#	This program is free software; you can redistribute it and/or
#	modify it under the terms of the GNU General Public License
#	as published by the Free Software Foundation; either version 2
#	of the License, or (at your option) any later version.

############################################################################

package BigMac::QueryData;
use FileHandle;
use Exporter;

BEGIN 
{
    use vars qw($VERSION @ISA @EXPORT @EXPORT_OK);
    @ISA = qw(Exporter);
    @EXPORT = qw();
    @EXPORT_OK = qw();
    $VERSION = "Beta_Version"; 
}

END 
{ 
} 

# initialization routine
sub _initialize
{
	my $self = shift;
    	$self->openData();
}

# constructor
sub new
{
	my $class = shift;
	my $self = {};
	bless $self, $class;
    	$self->_initialize ();
	return $self;
}

# to load the data
sub openData
{
    	my $self = shift;
	my $file= "/home/cs2/tpederse/CS8761/BigMac/bigmac.sgml";
	open(FILE,$file);
	$old_word="#";
	while($line=<FILE>)
	{
		chomp $line;
		if($line=~/<HEAD>([\w\s]+)</)
		{
			$word=$1;
			if(defined $dictionary{$word})
			{
				$dictionary{$word}.=$line;
			}
			else
			{
				$dictionary{$word}=$line;
			}
		}
		else
		{
			if(defined $word)
			{
				$dictionary{$word}.=$line;
			}
		}
	}
}

# displays all the sense definitions for the given word
sub get_sense_defs
{
    my( @senses ) = ();
	$word=$_[1];
	$entry=$dictionary{$word};
#	print "Sense Definitions for '$word'\n";
	while($entry=~/<DTEXT>([\w\s.;:"'\-,\?\(\)\{\}]+)</)
	{
		push @senses, $1;
		$entry=$';
	}
	return @senses;
}
# displays the illustrative phrases for the word
sub get_phrases
{
    my( @phrases ) = ();
	$word=$_[1];
        $entry=$dictionary{$word};
#        print "Illustrative Phrases for '$word'\n";
        while($entry=~/<IP>([\w\s\.;:"\-',\?\(\)\{\}]+)</)
        {
                push @phrases,$1;
                $entry=$';
        }
	return @phrases;
}

# shows the topics/fields in which the word comes
sub get_topics
{
    my( @topics ) = ();
	$word=$_[1];
        $entry=$dictionary{$word};
	return () if !defined $entry;
#        print "Topics in which '$word' occurs\n";
        while($entry=~/<NPLABEL>([\w\s\.;:'\-",\?\(\)\{\}]+)</)
        {
                push @topics,$1;
                $entry=$';
        }
	return @topics;
}
# displays the common usage of the word
sub get_usage
{
    my( @usages ) = ();
	$word=$_[1];
        $entry=$dictionary{$word};
#        print "Common Usage for '$word'\n";
        while($entry=~/<LABEL>([\w\s\.;:"\-',\?\(\)\{\}]+)</)
        {
                push @usages,$1;
                $entry=$';
        }
	return @usages;
}	

# displays the forms for the word
sub get_forms
{
    my( @forms ) = ();
	my $word=$_[1];
        $entry=$dictionary{$word};
#        print "Inflections for '$word'\n";
        while($entry=~/<INF>([\w\s\.;:"\-',\?\(\)\{\}]+)</)
        {
                push @forms,$1;
                $entry=$';
        }
	return @forms;
}

# diplays the POS information given a word
sub get_pos
{
    my( @poses ) = ();
	$word=$_[1];
        $entry=$dictionary{$word};
#        print "Parts of speech for '$word'\n";
	return () if !defined $entry;

        while($entry=~/<POS>([\w\s\.;:\"\-\',\?\(\)\{\}]+)</)
        {
                push @poses,$1;
                $entry=$';
        }

#        print "Poses for $word are @poses\n";
	return @poses;
}

#get all POS tags
sub get_all_pos
{
    my( %pos_tags ) = ();
	foreach $word (keys %dictionary)
	{
        	$entry=$dictionary{$word};
	        while($entry=~/<POS>([\w\s\.;:"\-',\?\(\)\{\}]+)</)
        	{
			$pos_tags{$1}++;
	                $entry=$';
        	}
	}
        return %pos_tags;
}

#get all topics 
sub get_all_topics
{
    my( %topics ) = ();
        foreach $word (keys %dictionary)
        {
                $entry=$dictionary{$word};
                while($entry)
		{
			if($entry=~/<LABEL>([\w\s\.;:"\-',\?\(\)\{\}]+)</)
                	{
                        	$topics{$1}++;
                        	$entry=$';
				next;
                	}
			last;
		}
        }
        return %topics;
}

# displays the pronounciation for the word
sub get_pron
{
    my( @prons ) = ();
	$word=$_[1];
        $entry=$dictionary{$word};
#        print "Pronounciations for '$word'\n";
        while($entry=~/<PRN TYPE=SAY>([\w\s\.;:"'\-,\?\(\)\{\}]+)</)
        {
                push @prons,$1;
                $entry=$';
        }
	return @prons;
}

# displays the variants for the word
sub get_variants
{
    my( @variants ) = ();
	$word=$_[1];
        $entry=$dictionary{$word};
#        print "Variants for '$word'\n";
        while($entry=~/<VAR>([\w\s\.;:"\-',\?\(\)\{\}]+)</)
        {
                push @variants,$1;
                $entry=$';
        }
	return @variants;
}

# displays he abbreviations for the word
sub get_abbreviations
{
    my( @abbrevs ) = ();
        $word=$_[1];
        $entry=$dictionary{$word};
#        print "Abbrevations for '$word'\n";
        while($entry=~/<ABBREV>([\w\s\.;:"\-',\?\(\)\{\}]+)</)
        {
                push @abbrevs, $1;
                $entry=$';
        }
	return @abbrevs;
}

# displays the prepositions which come with the word
sub get_contexts
{
    my( @contexts ) = ();
        $word=$_[1];
        $entry=$dictionary{$word};
#        print "Common prepositions coming with '$word'\n";
        while($entry=~/<CW>([\w\s\.;:"\-',\?\(\)\{\}]+)</)
        {
                push @contexts,$1;
                $entry=$';
        }
	return @contexts;
}

# displays the cross references for the word
sub get_references
{
    my( @refs ) = ();
        $word=$_[1];
        $entry=$dictionary{$word};
#        print "Cross References for $word\n";
        while($entry=~/<CFTEXT>([\w\s\.;:'\-",\?\(\)\{\}]+)</)
        {
                push @refs,$1;
                $entry=$';
        }
	return @refs;
}

# lists all the words for a given topic
sub get_words_for_topic
{
    my( @words ) = ();
    	$topic = $_[1];
	$topic=~tr/A-Z/a-z/;
#    	print "Words for a topic : $topic\n";
    	foreach $word(keys %dictionary)
    	{
    		$entry = $dictionary{$word};
	       	while($entry=~/<NPLABEL>([\w\s\.;:"\-',\?\(\)\{\}]+)</)
        	{
        		$temp = $1;
			$temp=~tr/A-Z/a-z/;
	        	$entry = $';
        	     	if($temp eq $topic)
               		{
                		push @words,$word;
     	          	}
        	 }
    	}
	return @words;
}

#get thes code
sub get_thes_code
{
    my( @codes ) = ();
	$word= $_[1];
        $entry = $dictionary{$word};
	return () if !defined $entry;
        while($entry=~/<THES>([^>^<]+)</)
        {
                $code=$1;
		$entry=$';
		push @codes,$code;
	}
	return @codes;
}

# displays the thesaurus entry for the word
sub get_thes 
{
    my( @theses ) = ();
        $word= $_[1];
#        print "Thesaurus Entry for $word\n";
        $entry = $dictionary{$word};
	open(THES,"thes.data")||die"Error in opening thes.data\n";
        while($entry=~/<THES>([\w\s\.;:"\-',\?\(\)\{\}]+)</)
        {
		$code=$1;
		$entry=$';
		@code_parts=split(/\./,$code);
		$code_parts[0]=~s/0*([1-9]+)/$1/;
		$code_parts[1]=~s/0//g;
		$code_parts[2]=~s/0//g;
		$code=$code_parts[0].".".$code_parts[1].".".$code_parts[2];
		while($line=<THES>)
		{
			if($line=~/$code/)
			{
				$line=~s/.*\|(.*)/$1/;
				push @theses,$line;
			}
		}	
        }
	return @theses;
}

# displays all the words for a given POS
sub get_words_given_pos 
{
    my( @words ) = ();
    #undef @words;
	$pos = $_[1];
#	print "Words which have $pos as Part of Speech\n";
	foreach $word (keys %dictionary)
	{
       		$entry = $dictionary{$word};
		while($entry=~/<POS>([\w\s\.;:"\-',\?\(\)\{\}]+)</)
          	{
		           $temp = $1;
		           $entry = $';
		           if($temp =~ /$pos/)
		           {
                		 push @words,$word;
		           }
          	}
     	}
	return @words;
}

# displays all nouns
sub get_nouns
{
    my( @nouns ) = ();
#        print "All Nouns - \n";
        foreach $word (keys %dictionary)
        {
                $entry = $dictionary{$word};
                while($entry=~/<POS>([^<]+)</)
                {
                           $temp = $1;
                           $entry = $';
                           if($temp =~ /noun/)
                           {
                                 push @nouns,$word;
                           }
                }
        }	
	return @nouns;
}

# displays all verbs
sub get_verbs
{
    my( @verbs ) = ();
#        print "All Verbs - \n";
        foreach $word (keys %dictionary)
        {
                $entry = $dictionary{$word};
                while($entry=~/<POS>([\w\s\.;:"\-',\?\(\)\{\}]+)</)
                {
                           $temp = $1;
                           $entry = $';
                           if($temp =~ /verb/)
                           {
                                 push @verbs,$word;
                           }
                }
        }
	return @verbs;
}

# displays all the adjectives
sub get_adjs
{
    my( @adjs ) = ();
#        print "All Adjectives - \n";
        foreach $word (keys %dictionary)
        {
                $entry = $dictionary{$word};
                while($entry=~/<POS>([\w\s\.;:"\-',\?\(\)\{\}]+)</)
                {
                           $temp = $1;
                           $entry = $';
                           if($temp =~ /adj/)
                           {
                                 push @adjs,$word;
                           }
                }
        }
	return @adjs;
}

#all nouns starting with a given letter
sub get_nouns_given_letter
{
    my( @nouns ) = ();
    # undef @nouns;
	$letter=$_[1];
#        print "All Nouns starting with - $letter\n";
        foreach $word (keys %dictionary)
        {
		if($word=~/^$letter/)
		{
                	$entry = $dictionary{$word};
	                while($entry=~/<POS>([\w\s\.;:"\-',\?\(\)\{\}]+)</)
        	        {
                	           $temp = $1;
                        	   $entry = $';
	                           if($temp =~ /noun/)
        	                   {
                	                 push @nouns,$word;
                        	   }
                	}
		}
        }
	return @nouns;
}
sub get_verbs_given_letter
{
    my( @verbs ) = ();
    # undef @verbs;
        $letter=$_[1];
#        print "All Verbs starting with - \n";
        foreach $word (keys %dictionary)
        {
                if($word=~/^$letter/)
                {
                        $entry = $dictionary{$word};
                        while($entry=~/<POS>([\w\s\.;:"\-',\?\(\)\{\}]+)</)
                        {
                                   $temp = $1;
                                   $entry = $';
                                   if($temp =~ /verb/)
                                   {
                                         push @verbs,$word;
                                   }
                        }
                }
        }
        return @verbs;
}
sub get_adjs_given_letter
{
    my( @adjs ) = ();
    # undef @adjs;
        $letter=$_[1];
#        print "All Adjectives starting with - \n";
        foreach $word (keys %dictionary)
        {
                if($word=~/^$letter/)
                {
                        $entry = $dictionary{$word};
                        while($entry=~/<POS>([\w\s\.;:"\-',\?\(\)\{\}]+)</)
                        {
                                   $temp = $1;
                                   $entry = $';
                                   if($temp =~ /adj/)
                                   {
                                         push @adjs,$word;
                                   }
                        }
                }
        }
        return @adjs;
}

sub get_words_given_words_in_def
{
	$class=shift;
	my @words=@_;
	foreach $entry (keys %dictionary)
	{
		$word_entry=$dictionary{$entry};
		@word_defs=split(/<\/DEF>/,$word_entry);
		foreach $def(@word_defs)
		{
			$match=1;
			foreach $word(@words)
			{
				if(!($def=~/<DTEXT>[^<]*$word[^<]*</))
				{
					$match=0;
					last;	
				}
			}
			if($match==1)
			{
				if($def=~/<POS>([^<]+)</)		
				{
					$pos=$1;
				}
				if($def=~/<THES>([^<]+)</)
                                {
                                       $thes=$1;
                                }
				$output="<head=$entry>";
				if(defined $pos)
				{
					$output.="<pos=$pos>";
				}
				if(defined $thes)
				{
					$output.="<thes=$thes>";
				}
				push @return_arr,$output;
			}
		}
	}
	return @return_arr;
}
sub get_words_given_word_thes
{
    my $word=$_[1];
	$entry=$dictionary{$word};
	while($entry=~/<THES>([^<]+)</)
	{
		$thes=$1;
		push @all_theses,$thes;
		$entry=$';
	}
	foreach $dict_entry (keys %dictionary)
	{
		$word_entry=$dictionary{$dict_entry};
		while($word_entry=~/<THES>([^<]+)</)
		{
			$word_thes=$1;
			$word_entry=$';
			foreach $thes_code (@all_theses)
			{
				if($word_thes eq $thes_code)
				{
					push @{$semantic{$thes_code}},$dict_entry;
				}
			}
		}
	}
	foreach $thes_code(keys %semantic)
	{
		@words_with_thes=@{$semantic{$thes_code}};
		$string=join "\t",@words_with_thes;
		print "\nAll words belonging to the Semantic class $thes_code - \n",$string;
	}
}
1;

__END__
#################
# Documentation #
#################

=head1 NAME

	QueryData - Perl Module to provide an interface to  
                    Macquarie Dictionary (aka Big Mac) database

=head1 SYNOPSIS

  use QueryData;

  my $bm = QueryData->new;

##############################################################

=head2 How to call various subroutines 

(1)	Sense Definitions of a given word

 # calling sense_defs subroutine
 # input is the word
 print "\nSenses for task - \n",join " ",
 $bm->get_sense_defs("task");

(2)	Illustrative examples for a given word

 # calling the phrases subroutine which returns
 # the illustrative examples or phrases for the word
 print "\nIllustrative phrases for taste - \n",join " ",
 $bm->get_phrases("taste");


(3)	All the topics associated with a given word

 # lists the topics associated with the word
 print "\nTopics for tax - \n",join " ",$bm->get_topics("tax");


(4)	Common usage of a word

 # displays the usage of the word
 print "\nUsage for accelerator - \n", join " ",
 $bm->get_usage("accelerator");


(5)	Forms of a word

 # displays various forms of the word
 print "\nForms of taxi - \n",join " ",$bm->get_forms("taxi");


(6)	POS information for a word

 # displays the POS information of the word
 print "\nPOS for target - \n",join " ",$bm->get_pos("target");


(7)	How to pronounce a word

 # displays the pronounciation
 print "\nPronounciations for accelerator - \n",join " ",
 $bm->get_pron("accelerator");


(8)	Variants of a word

 # lists all the variants
 print "\nVariants of accelerator - \n",join " ",
 $bm->get_variants("accelerator");


(9)	Common Abbreviations of a word

 # displays the abbreviations for the word
 print "\nAbbreviations for Amplitude Modulation- \n",join " ",
 $bm->get_abbreviations("amplitude modulation");


(10)	Common Prepositions which co-occur with a word

 # displays the prepositions which come with the word
 print "\nCommon Prepositions coming with angry - \n", join " ",
 $bm->get_contexts("angry");


(11)	Cross-References of a Word

 # displays the cross references
 print "\nCross-References with area school- \n",join " ",
 $bm->get_references("area school");


(12)	All the words under a given topic/subject

 # lists all the words with the given topic
 print "\nAll words for Topic Computers - \n",join " ",
 $bm->get_words_for_topic("computers");


(13)	Thesaurus entry for a word

 # displays the thesaurus entry for the word
 print "\nThesaurus Entry for accelerator - \n",join " ",
 $bm->get_thes("accelerator");


(14)	All words given a POS

 # displays the words with given POS
 #print "\nWords with POS = noun \n", join " ",
 $bm->get_words_given_pos("noun");


(15)	All adjectives

 # displays all the adjectives
 #print "\nAll adjectives - \n", join " ",$bm->get_adjs();

(16)	All Nouns starting with a given alphabet

 # displays all the nouns with a given letter
 print "\nNouns starting with f - \n",join " ",
 $bm->get_nouns_given_letter("f");

(17)	All Adjectives starting with a given alphabet

 # displays all the adjectives with a given letter
 print "\nAdjectives starting with f - \n",join " ",
 $bm->get_adjs_given_letter("f");

(18)	All Verbs starting with a given alphabet

 # displays all the verbs with a given letter
 print "\nVerbs starting with f - \n",join " ",
 $bm->get_verbs_given_letter("f");


(19)	All words using a given set of words in their senses

 #all words having given words in definitions
 print "All words which have given set of words in their definitions",
 $bm->get_words_given_words_in_def("book");


(20)	All words under the same semantic code

 #all words sharing the same semantic code of the given word
 print "All words which have the same semantic code as the given word -",
 $bm->get_words_given_word_thes("dog");

(21)	All Topics which appear in BigMac 

 %topics=$bm->get_all_topics();	
 Returns all the topics in BigMac

(22)	All POS Tags which appear in BigMac
 %pos=$bm->get_all_pos();
 Returns all the POS Tags in BigMac

(24)	THES CODE Given a word
 #returns the THES Code for a word
 print "\nThes Codes for 'good'-\n ", join " ", $bm->get_thes_code("good");

 #################################################################

=head1 DESCRIPTION

 QueryData Module provides an interface to the Macquarie Dictionary 
 (aka Big Mac) database. It needs the BigMac Dictionary and the 
 Thesaurus file thes.data in the current directory. 
 
=head2 QUERYING THE DATABASE

 It provides the user with the some useful information for a given a 
 word like 

 (1)	All senses associated with the word
 	Accepts a word and returns a list of all 
	possible senses for the word
 (2)	Illustrative Phrases
	Accepts a word and returns a list of 
	illustrative phrases for the word
 (3)	All the Topics associated with the word
	Accepts a word and returns a list of 
	topics under which the word can be found
 (4)	Common Usage of the word
	Accepts a word and returns a list of usage 
	of the word
 (5)	Different forms of the word
	Accepts a word and returns a list of forms 
	listed in BigMac for the word
 (6)	Different POS tags applicable
	Accepts a word and returns a list of 
	possible POS tags of the word
 (7)	How to pronounce the word
	Accepts a word and returns the pronounciation 
	of the word
 (8)	Variants 
	Accepts a word and returns a list of variants 
	of the word
 (9)	Common Prepositions coming with the word
	Accepts a word and returns a list of prepositions 
 	coming with the word
 (10)	Cross-References for the word
	Accepts a word and returns a list of references 
	of the word
 (11)	List of all the words under a given topic
	Accepts a topic and returns a list of words 
	coming under it
 (12)	Thesaurus Entry for the word
	Accepts a word and returns the thesaurus entry 
	of the word
 (13)	All the words for given POS
	Accepts a POS tag and returns a list of words 
	with that tag
 (14)	All nouns
	Accepts nothing and returns all nouns in BigMac
 (15)	All Verbs
 	Accepts nothing and returns all verbs in BigMac
 (16)	All adjectives
	Accepts nothing and returns all adjectives in BigMac
 (17)	All nouns starting with a given letter
	Accepts a letter and returns all nouns starting with 
	the letter
 (18)	All adjectives starting with a given letter
	Accepts a letter and returns all adjectives starting 
	with the letter
 (19)	All verbs starting with a given letter
	Accepts a letter and returns all verbs starting 
	with the letter
 (20)	All words which have the given set of words in their 
	definition 
	Accepts a word and returns all the words which have 
	the given word in their definitions
 (21)	All words having the same semantic code as the given 
	word
	Accepts a word and returns all the words which have 
	the same semantic code as that of the given word
 (22)	All POS tags available in BigMac
	Accepts nothing and returns the list of POS tags found.
 (23)	All topics 
	Accepts nothing and returns the list of all LABELs
 (24)	THES code for a word
	Accepts a word and returns its THES CODES

	see test.pl for some sample usages and examples
 ##############################################################

=head1 COPYRIGHT

 Copyright (c) 2002,
 Amine Abou-Rjeily,University of Minnesota, Duluth
 abou0013@d.umn.edu 
 Amruta Purandare, University of Minnesota, Duluth
 pura0010@d.umn.edu
 Sam Storie, University of Minnesota, Duluth
 sstorie@d.umn.edu
 Suchitra Goopy, University of Minnesota, Duluth
 goop0001@d.umn.edu 
 Ted Pedersen, University of Minnesota, Duluth
 tpederse@umn.edu

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to

 The Free Software Foundation, Inc.,
 59 Temple Place - Suite 330,
 Boston, MA  02111-1307, USA.

 ###############################################################
=head1 SEE ALSO
 MacQuarie References
 http://www.clres.com/macq-dimap-contents.html
 http://www.clres.com/trec9.html
 http://www.clres.com/macq-clr.html
 CPAN site
 http://search.cpan.org/
=cut
