# Before `make install' is performed this script should be runnable with
# `make test'. After `make install' it should work as `perl 1.t'

#########################

# change 'tests => 1' to 'tests => last_test_to_print';

#use Test::More tests => 7;
#BEGIN { use_ok('LDOCE::Interface') };

use Test::Simple tests => 20;
use LDOCE::Interface;

#########################

# Insert your test code below, the Test::More module is use()ed here so read
# its man page ( perldoc Test::More ) for help writing this test script.

# Test 1
my $ldoce = LDOCE::Interface->new;
ok( defined($ldoce) && ref $ldoce eq 'LDOCE::Interface', 'new() works');

# Grab some information about a known word and test it
$word = "sample";

# Test 2
# Try grabbing some raw SGML
#
$data = $ldoce->QuerySGML($word);
ok( defined($data), 'QuerySGML() works');

# Test 3
# Get some head information
#
@head = $ldoce->GetHead($word);
$href = $head[0];
ok( $href->{hwd} eq 'sample', 'head information is correct');

# Test 4
# Get some sense information
#
@senses = $ldoce->GetSenses($word);
$href = $senses[1];
ok( $href->{activ} eq 'TRY STH TO SEE IF IT IS GOOD', 'Sense data is correct');

# Test 5
# Query for a list of nouns
@nouns = $ldoce->QueryByPos('n');
ok( @nouns, 'Test for list of nouns successful');

# Test 6
# Query for a list of words in the 'DFF' category
#
@dffWords = $ldoce->QuerySubjectCode('DFF', '0');
ok( @dffWords, 'Test for list of subject codes ok');

# Test 7
# Get the words related to 'canine'
#
%relatedWords = $ldoce->QueryRelated( 'canine' );
ok( %relatedWords, 'Test for related words ok' );

# Test 8
# Get the senses of a list of words [canine, drunk]
#
%defns = $ldoce->GetSensesMultiple( ('canine', 'drunk') );
ok( %defns, 'Test for multiple word senses ok' );

# Test 9
# Get all the subject codes
#
@allSubjectCodes = $ldoce->GetSubjectCodes();
ok( @allSubjectCodes, 'Test to get all subject codes ok' );

# Test 10
# Let us get all the POS codes
#
@allPosCodes = $ldoce->GetPosCodes();
ok( @allPosCodes, 'Test to get all the POS codes ok' );

# Test 11
# Let us query by subject code 'A' and word 'pulp'. There should only be one
# sense: books, magazines, films etc that are of poor quality or are badly written
#
@allSenses = $ldoce->QuerySubjectCodeByWord( 'pulp', 'A' );
ok( @allSenses == 1, "Test for subject code 'A' and word 'pulp' ok" );

# Test 12
# Query for a list of verbs
#
@verbs = $ldoce->QueryByPos('v');
ok( @verbs, 'Test for list of verbs successful');

# Test 13
# Let us test for abnormal cases now
# Query for a non-existent subject code
#
@emptyArray = $ldoce->QuerySubjectCode( 'sadsa' );
ok( @emptyArray == 0, 'Test for unknown subject code ok' );

# Test 14
# Test for undefined POS
#
@emptyArray = $ldoce->QueryByPos( 'dasdas' );
ok( @emptyArray == 0, 'Test for unknown POS ok' );

# Test 15
# Query senses for an unknown word
# 
@noSenses = $ldoce->GetSenses( 'dassa' );
ok( @noSenses == 0, 'Test for senses of unknown word ok' );

# Test 16
# Query for empty word and subject code
#
@noSenses = $ldoce->QuerySubjectCodeByWord( '', '' );
ok( @noSenses == 0, 'Test for empty word and subject code ok' );

# Test 17
# Query for an empty list
#
%emptyHash = $ldoce->GetSensesMultiple();
ok( !%emptyHash, 'Test for empty list for senses ok' );

# Test 18
# Get related words for a word that does not exist
#
%emptyHash = $ldoce->QueryRelated('dasda');
ok( !%emptyHash, 'Test for related words for unknown word ok' );

# Test 19
# Get the Head for a word not in the dictionary
#
@emptyArr = $ldoce->GetHead( 'dasdas' );
ok( @emptyArr == 0, 'Test for head of unknown word ok' );

# Test 20
# Pass undef to function getSenses()
#
@noSenses = $ldoce->GetSenses( undef );
ok( @noSenses == 0, 'Test for undef handling of GetSenses() ok' );